<?php

/* UPC-A as specified by http://www.uc-council.org/reflib/01302/d36-2.htm */

$conversion_table = array(0x0D, 0x19, 0x13, 0x3D, 0x23, 0x31, 0x2F, 0x3B, 0x37, 0x0B);


function dec_to_upca($upca_num_str) {
  if(strlen($upca_num_str) > 11) $upca_num_str = substr($upca_num_str,0,11);
  while(strlen($upca_num_str) < 11) $upca_num_str = "0" . $upca_num_str;
  $upca_num_str .= calculate_checksum($upca_num_str);
  return($upca_num_str);
}


function calculate_checksum($upca_num_str) {
  $even_pos = $odd_pos = 0;
  for($i = 0; $i < 11; $i++) {
    $curr_num = (int)(substr($upca_num_str, $i, 1));
    if($i %2 == 0) $even_pos += $curr_num;
    else $odd_pos += $curr_num; 
  }
  $checksum = 10 - (((3 * $even_pos) + $odd_pos) % 10);
  if($checksum == 10) $checksum = 0;
  return($checksum);
}


function convert_to_upca_bitstring($upc_num_str) {
  global $conversion_table;
  $bit_string = "101";                    /* start string */
  $upc_num_str = dec_to_upca($upc_num_str);
  $input_length = strlen($upc_num_str);
  for($i = 0; $i < $input_length; $i++) { 
    if($i == 6) $bit_string .= "01010";   /* center bar pattern */
    $curr_num = $conversion_table[(int)(substr($upc_num_str,$i,1))];
    if($i > 5) $curr_num = ~$curr_num & 0x7f;
    $tmp_string = decbin($curr_num);
    while(strlen($tmp_string) < 7) $tmp_string = "0" . $tmp_string;
    $bit_string .= $tmp_string;
  }
  $bit_string .= "101";                   /* stop string */
  return($bit_string);
}


function draw_upca_png($upc_num_str, $width_scale, $height, $border_size) {
  $upc_num_str = dec_to_upca($upc_num_str);
  $bit_string = convert_to_upca_bitstring($upc_num_str);
  $width = $width_scale * (strlen($bit_string) + ($border_size * 2));
  $upca_image = ImageCreate($width, $height);

  /* Constants */
  $black = ImageColorAllocate($upca_image,0,0,0);
  $white = ImageColorAllocate($upca_image,255,255,255);

  ImageFilledRectangle($upca_image, 0,0,$width,$height,$white);

  $curr_pos = 0;
  for($i = 0; $i < strlen($bit_string); $i++) {
    if(substr($bit_string,$i,1) == "0") $bar_color = $white;
    else $bar_color = $black;
    ImageFilledRectangle($upca_image, ($i+$border_size)*$width_scale, 5, (($i+$border_size)*$width_scale) + $width_scale - 1, $height - 10, $bar_color);
  }

  $font_size = 4;
  $font_width = ImageFontWidth($font_size);
  $font_height = ImageFontHeight($font_size);
  $font_top = $height - $font_height;
  $font_bot = $height;

  /* Digit 0 */
  ImageFilledRectangle($upca_image, $width / 2 - (7 * $font_width + 2 * $width_scale), $font_top, $width / 2 - (6 * $font_width + 2 * $width_scale), $font_bot, $white);
  ImageString($upca_image, $font_size, $width / 2 - (7 * $font_width + 2 * $width_scale), $font_top , substr($upc_num_str,0,1), $black);

  /* Digits 1 - 5 */
  ImageFilledRectangle($upca_image, $width / 2 - (5 * $font_width + 2 * $width_scale), $font_top, ($width / 2) - (2 * $width_scale), $font_bot, $white);
  ImageString($upca_image, $font_size, $width / 2 - (5 * $font_width + 2 * $width_scale), $font_top, substr($upc_num_str,1,5), $black);

  /* Digits 6 - 10 */
  ImageFilledRectangle($upca_image, $width / 2 + (2 * $width_scale), $font_top, ($width / 2) + (5 * $font_width + (2 * $width_scale)), $font_bot, $white);
  ImageString($upca_image, $font_size, ($width / 2) + (3 * $width_scale), $font_top, substr($upc_num_str,6,5), $black);

  /* Digit 11 */
  ImageFilledRectangle($upca_image, $width / 2 + (6 * $font_width + 2 * $width_scale), $font_top, $width / 2 + (7 * $font_width + 2 * $width_scale), $font_bot, $white);
  ImageString($upca_image, $font_size, $width / 2 + (6 * $font_width + 2 * $width_scale) ,$font_top, substr($upc_num_str,11,1), $black);


  ImagePng($upca_image);  
}

?>
